@@ -1,4 +1,4 @@
-from biblemate import config
+from biblemate import config, BIBLEMATE_VERSION
 from prompt_toolkit.layout import Layout, HSplit
 from prompt_toolkit.widgets import Frame, Label
 from prompt_toolkit.styles import Style, merge_styles
@@ -12,19 +12,14 @@ from prompt_toolkit.lexers import PygmentsLexer
 from pygments.lexers.markup import MarkdownLexer
 from prompt_toolkit.styles import style_from_pygments_cls
 from pygments.styles import get_style_by_name
-from agentmake import edit_file, readTextFile, writeTextFile
+from agentmake import DEFAULT_TEXT_EDITOR, edit_file, readTextFile, writeTextFile
+from agentmake.etextedit import launch_async
+from typing import Any, Optional
 import os
 
 
-async def getTextArea(input_suggestions:list=None, default_entry="", title="", multiline:bool=True):
+async def getTextArea(input_suggestions:list=None, default_entry="", title="", multiline:bool=True, completion:Optional[Any]=None, scrollbar:bool=True, read_only:bool=False):
     """Get text area input with a border frame"""
-    
-    def edit_temp_file(initial_content: str) -> str:
-        config.current_prompt = ""
-        temp_file = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "temp", "edit.md")
-        writeTextFile(temp_file, initial_content)
-        edit_file(temp_file)
-        return readTextFile(temp_file).strip()
 
     config.current_prompt = "" # reset config.current_prompt
     completer = FuzzyCompleter(WordCompleter(input_suggestions, ignore_case=True)) if input_suggestions else None
@@ -51,13 +46,77 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
         style="class:textarea",
         lexer=PygmentsLexer(MarkdownLexer),
         multiline=multiline,
-        scrollbar=True,
-        completer=completer,         # <-- attach completer here
-        complete_while_typing=False,       # only trigger when requested
+        scrollbar=scrollbar,
+        read_only=read_only,
+        completer=completer,
+        complete_while_typing=config.auto_suggestions,
         focus_on_click=True,
         wrap_lines=True,
     )
 
+    def unpack_text_chunks(_):
+        openai_style = True if config.backend in ("azure", "azure_any", "custom", "deepseek", "github", "github_any", "googleai", "groq", "llamacpp", "mistral", "openai", "xai") else False
+        first_event = True
+        chat_response = ""
+        for event in completion:
+            # RETRIEVE THE TEXT FROM THE RESPONSE
+            if event is None:
+                continue
+            elif openai_style:
+                # openai
+                # when open api key is invalid for some reasons, event response in string
+                if isinstance(event, str):
+                    answer = event
+                elif hasattr(event, "data") and hasattr(event.data, "choices"): # mistralai
+                    try:
+                        answer = event.data.choices[0].delta.content
+                    except:
+                        answer = None
+                elif hasattr(event, "choices") and not event.choices: # in case of the 1st event of azure's completion
+                    continue
+                else:
+                    answer = event.choices[0].delta.content or ""
+            elif hasattr(event, "type") and event.type == "content-delta" and hasattr(event, "delta"): # cohere
+                answer = event.delta.message.content.text
+            elif hasattr(event, "delta") and hasattr(event.delta, "text"): # anthropic
+                answer = event.delta.text
+            elif hasattr(event, "content_block") and hasattr(event.content_block, "text"):
+                answer = event.content_block.text
+            elif str(type(event)).startswith("<class 'anthropic.types"): # anthropic
+                continue
+            elif hasattr(event, "message"): # newer ollama python package
+                answer = event.message.content
+            elif isinstance(event, dict):
+                if "message" in event:
+                    # ollama chat
+                    answer = event["message"].get("content", "")
+                else:
+                    # llama.cpp chat
+                    answer = event["choices"][0]["delta"].get("content", "")
+            elif hasattr(event, "text"):
+                # vertex ai, genai
+                answer = event.text
+            else:
+                #print(event)
+                answer = None
+            # STREAM THE ANSWER
+            if answer is not None:
+                if first_event:
+                    first_event = False
+                    answer = answer.lstrip()
+                # update the chat response
+                chat_response += answer
+                # display the chunk in the text area
+                text_area.buffer.insert_text(answer)
+                text_area.buffer.cursor_position = len(text_area.text)
+
+    def edit_temp_file(initial_content: str) -> str:
+        config.current_prompt = ""
+        temp_file = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "temp", "edit.md")
+        writeTextFile(temp_file, initial_content)
+        edit_file(temp_file)
+        return readTextFile(temp_file).strip()
+
     # Layout: include a CompletionsMenu
     root_container = HSplit(
         [
@@ -66,7 +125,7 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
                 title=title,
             ),
             Label(
-                "[Ctrl+S] Send [Ctrl+K] Help",
+                "[Ctrl+S] Send [Ctrl+Q] Exit" if title else "[Ctrl+S] Send [Ctrl+Y] Help",
                 align=WindowAlign.RIGHT,
                 style="fg:grey",
             ),
@@ -79,7 +138,35 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
     
     # Create key bindings
     bindings = KeyBindings()
+    config.cursor_position = 0
     
+    if not title: # these shortcuts are irrelevant for review or configuration prompts
+        # help
+        @bindings.add("c-y")
+        def _(event):
+            config.current_prompt = text_area.text
+            event.app.exit(result=".help")
+        # get ideas
+        @bindings.add("c-g")
+        def _(event):
+            config.current_prompt = text_area.text
+            event.app.exit(result=".ideas")
+        # new chat
+        @bindings.add("c-n")
+        def _(event):
+            config.current_prompt = text_area.text
+            event.app.exit(result=".new")
+
+    # launch editor
+    @bindings.add("c-p")
+    def _(event):
+        config.cursor_position = text_area.buffer.cursor_position
+        config.current_prompt = text_area.text
+        event.app.exit(result=".editprompt")
+    # exit
+    @bindings.add("c-q")
+    def _(event):
+        event.app.exit(result=".exit")
     # submit
     @bindings.add("escape", "enter")
     @bindings.add("c-s")
@@ -95,30 +182,6 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
             event.app.exit(result=text_area.text.strip())
         else:
             text_area.buffer.newline()
-    # help
-    @bindings.add("c-y")
-    def _(event):
-        config.current_prompt = text_area.text
-        event.app.exit(result=".help")
-    # launch editor
-    @bindings.add("c-p")
-    def _(event):
-        config.current_prompt = text_area.text
-        event.app.exit(result=".editprompt")
-    # get ideas
-    @bindings.add("c-g")
-    def _(event):
-        config.current_prompt = text_area.text
-        event.app.exit(result=".ideas")
-    # new chat
-    @bindings.add("c-n")
-    def _(event):
-        config.current_prompt = text_area.text
-        event.app.exit(result=".new")
-    # exit
-    @bindings.add("c-q")
-    def _(event):
-        event.app.exit(result=".exit")
     # insert four spaces
     @bindings.add("s-tab")
     def _(event):
@@ -154,14 +217,22 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
         #clipboard=PyperclipClipboard(), # not useful if mouse_support is not enabled
         #mouse_support=True, # If enabled; content outside the app becomes unscrollable
         full_screen=False,
+        after_render=unpack_text_chunks if completion is not None else None,
     )
     
-    # Run async
+    # Run the app
     result = await app.run_async()
-
+    print()
+    # edit in full editor
     while result == ".editprompt":
-        text_area.text = edit_temp_file(config.current_prompt)
+        if DEFAULT_TEXT_EDITOR == "etextedit":
+            text_area.text = await launch_async(input_text=config.current_prompt, exitWithoutSaving=True, customTitle=f"BibleMate AI [{BIBLEMATE_VERSION}]", startAt=config.cursor_position)
+        else:
+            text_area.text = edit_temp_file(config.current_prompt)
+        config.current_prompt = ""
+        # Run the non-full-screen text area again
         result = await app.run_async()
-    
+        print()
+    # return the text content
     return result
 
